/*
**  Copyright (C) 1993-1996, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/
/*
 * 
 * Author: Jens Burkert	/ Andrew Hewett		Created: 11-08-93 
 * 	   Oldenburg University
 *	   Germany
 *
 * Module: dispuser
 * 
 * Purpose:
 * This file contains the interface to routines which provide 
 * user side access to the CTN display.  The implementation
 * translates these function calls into messages to send to
 * the CTN Display Process.
 * 
 * These routines are intended to be called from the CTN or from
 * the TI so that visitors to the RSNA'93 DICOM Demonstration
 * can see what activities are taking place.
 *
 * Module Prefix: DSU_ 
 * 
 * Last Update:		hewett
 * Revision:		1.3
 * Status:		Exp
 *
 * dispuser.h,v 1.3 1996/04/25 16:24:00 hewett Exp 
 * 
 * dispuser.h,v
 * Revision 1.3  1996/04/25 16:24:00  hewett
 * Added DSU_setTraceLevel() function to programatically enable/disable
 * printf tracing.
 *
 * Revision 1.2  1996/04/18 15:29:48  meichel
 * Version for CAR'96
 *
 * Revision 1.1  1996/03/28 17:08:00  meichel
 * first commit with new file structure
 *
 * Revision 2.1  1995/02/02 12:50:15  hewett
 * Corrected spelling of Ossietzky
 *
 * Revision 2.0  1994/11/17  09:45:39  meichel
 * Version 2.0 for RSNA 1994 demonstration.
 * New features: Metaheader support, new SOP classes, lots of bugs fixed.
 *
 * Revision 1.10  1994/03/17  09:28:37  hewett
 * Updated copyright notice.
 *
 * Revision 1.9  1993/11/16  12:26:15  jens
 * Added definitions for killassoc command.
 *
 * Revision 1.8  1993/10/26  16:06:37  jens
 * New command DSU_displayIndependant for sending w/o association
 *
 * Revision 1.7  1993/10/14  11:49:56  hewett
 * Added Euro Header
 *
 * Revision 1.6  1993/09/21  15:42:12  jens
 * Displaying images without association now with
 * DSU_displayImage(DSU_NoAssociation,filename)
 *
 * Revision 1.5  1993/08/31  18:44:38  jens
 * Additional function definition.
 *
 * Revision 1.4  1993/08/30  09:31:05  hewett
 * Removed extra comma in enum.
 *
 * Revision 1.3  1993/08/11  14:34:56  hewett
 * Added newline a end of file (CVS moans)
 *
 * Revision 1.2  1993/08/11  14:33:53  hewett
 * Added direction flag
 *
 * Revision 1.1  1993/08/11  13:59:47  hewett
 * Initial Revision
 *
 *
 */

#ifndef DISPUSER_H
#define DISPUSER_H

#include "osconfig.h"

/*
 * Enumerated Types
 */

typedef enum {
    DSU_CTNUser,		/* the user is the Central Test Node */
    DSU_TIUser			/* the user is the Telnet Initiator */
} DSU_UserType;		

typedef enum {
    DSU_Requestor,		/* an active requested association */
    DSU_Acceptor		/* a passive accepted association */
} DSU_AssociationType;

typedef enum {
    DSU_MainAssociation,	/* normal main association */
    DSU_SubAssociation,		/* sub-association (for MOVE) */
    DSU_NoAssociation           /* sending an Image without Association */
} DSU_AssociationLevel;

typedef enum {
    DSU_StoreOperation,
    DSU_FindOperation,
    DSU_MoveOperation,
    DSU_PrintOperation,
    DSU_KillOperation           /* only for internal use , kill an association */
} DSU_Operation;		/* the operation being performed */

typedef enum {
    DSU_StartOperation,		/* an operation has started */
    DSU_RunningOperation,	/* an operation is running */
    DSU_StopOperation		/* an operation has stopped */
} DSU_OperationState;

typedef enum {
    DSU_Incoming,		/* image receive */
    DSU_Outgoing		/* image send */
} DSU_Direction;

/*
 * Function Prototypes
 */

void DSU_setTraceLevel(int aTraceLevel);
/*
 * If (aTraceLevel > 0) causes trace information to be printed to stdout
 * during calls to DSU functions.
 */

/*
 * All the following routines return no success or failure information.
 * If any error occurs, the routines will print a message on the
 * standard error and return immediately.
 */

int DSU_initDisplay(DSU_UserType userType);
/*
 * Module initialization function.
 * returns true on successful initialisation, false otherwise.
 */

void DSU_setDisplayMode(int sendMessages);
/*
 * Set the display mode.
 *
 * If sendMessages is TRUE then subsequent function calls will
 * send messages to the Display Process.  If sendMessages is FALSE
 * then all subsequent function calls will return immediately (useful
 * for testing CTN and TI software without any display).
 * By default messages are sent to the Display Process (sendMessages==TRUE).
 */

void DSU_startAssociation(const char *peerHostName,
	const char *peerAETitle, const char *myAETitle, 
	DSU_AssociationType aType, DSU_AssociationLevel aLevel);
/*
 * Send a message to display that an association has started.
 *
 * Each association is normally handled within a separate process.
 * An exception is for MOVE sub-operations where a second association
 * is created within the same process so that images may be stored 
 * in the opposite direction.  The aLevel parameter indicates this
 * association level. 
 *
 * The direction of the association is indicated by aType.
 * myAETitle is the Application Entity title of the running process.
 *
 * The host name and peer AE titles are interpreted according to aType:
 * if aType == DSU_Requestor
 *	peerHostName is the host being called
 *	peerAETitle is the Application Entity title being called
 * if aType == DSU_Acceptor
 *	peerHostName is the host who called me
 *	peerAETitle is the Application Entity title who called me
 */

void DSU_stopAssociation(DSU_AssociationLevel aLevel);
/*
 * Send a message to indicate that an association has terminated.
 * 
 * the aLevel parameter specifies if a sub-association or the main
 * association has ended.
 */

void DSU_operationState(DSU_AssociationLevel aLevel,
	DSU_Operation op, DSU_OperationState state,
	DSU_Direction direction, int percentComplete);
/*
 * Send a message to indicate that an operation (op) has started, is
 * still running, or has stopped (state).
 *
 * The operation is taking place on the association specified by aLevel.
 *
 * The direction parameter is only relevant for Store operations and
 * specifies if an image is being received or send.
 *
 * The percentComplete parameter is only relevant for Store operations
 * with a state of DSU_runningOperation and specifies the amount of 
 * image data transmitted.  Normally, percentComplete will have a value
 * in the range 0..100 BUT may be greater that 100 if more image data
 * is transmitted that expected.  A value larger that 100 can be ignored.
 */

void DSU_displayImage(DSU_AssociationLevel aLevel,
	const char *imagePathName);
/*
 * Send a message to display the image stored in imagePathName.
 * The image was/will be transmitted on the association specified
 * by aLevel.
 *
 * Warning: in extreme situations the image file may not exist by the
 * time the Display Process tries to read the image file.
 * If this happens the Display Process should instead display a message
 * to indicate that the image has already been deleted.
 */

void DSU_displayIndependent(const char *ImageTitle,
	const char *imagePathName);
/*
 * Send a message to display the image stored in imagePathName.
 * The image was/will be transmitted without a association established
 * The ImageTitle String will be displayed above the Image
 *
 * Warning: in extreme situations the image file may not exist by the
 * time the Display Process tries to read the image file.
 * If this happens the Display Process should instead display a message
 * to indicate that the image has already been deleted.
 */

#endif /* DISPUSER_H */
