/*
**  Copyright (C) 1993/1994, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/
/*
** Service Class Executive (SCE)
** - Find Service Class Provider
**
** Author: 	Andrew Hewett
**		Universitaet Oldenburg, Germany
** Created:	08/1993
**
** Last Update:		hewett
** Update Date:		1996/09/27 08:46:21
** Source File:		/share/dicom/cvs-depot/dcmtk/imagectn/apps/scefind.cc,v
** CVS/RCS Revision:	1.3
** Status:		Exp
**
** CVS/RCS Log at end of file
*/

#include "osconfig.h"    /* make sure OS specific configuration is included first */

BEGIN_EXTERN_C
#include <stdio.h>
#include <string.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif
END_EXTERN_C

#include "dicom.h"
#include "imagectn.h"
#include "dimse.h"
#include "diutil.h"
#include "sce.h"
#include "imagedb.h"
#include "cnf.h"
#ifndef NODISPLAY
#include "dispuser.h"
#endif
#include "dcdeftag.h"

#include "scefind.h"


typedef struct {
    DB_Handle	*dbHandle;
    DIC_US	priorStatus;
    DIC_AE	ourAETitle;
} SCE_FindContext;

static void
addRetreiveAETitle(DcmDataset *rspIds, DIC_AE ourAETitle)
{
    /*
     * Since images are stored only by us (for RSNA'93 demo),
     * we must add in our AE Title to the response identifiers.
     * The DB cannot do this since it does not know our AE Title.
     */
    BOOLEAN ok;

    ok = DU_putStringDOElement(rspIds, DCM_RetrieveAETitle, ourAETitle);
    if (!ok) {
	errmsg("DO Error: adding Retreive AE Title");
    }
}



static void
findCallback(
	/* in */ 
	void *callbackData,  
	BOOLEAN cancelled, T_DIMSE_C_FindRQ *request, 
	DcmDataset *requestIdentifiers, int responseCount,
	/* out */
	T_DIMSE_C_FindRSP *response,
	DcmDataset **responseIdentifiers,
	DcmDataset **statusDetail)
{
    CONDITION dbcond;
    SCE_FindContext *context;
    DB_Status dbStatus;

    context = (SCE_FindContext*)callbackData;	/* recover context */

    dbStatus.status = context->priorStatus;
    dbStatus.statusDetail = NULL;
    
    if (responseCount == 1) {
        /* start the database search */
	if (verbose) {
	    printf("Find SCP Request Identifiers:\n");
	    requestIdentifiers->print();
        }
        dbcond = DB_startFindRequest(context->dbHandle, 
	    request->AffectedSOPClassUID, requestIdentifiers, &dbStatus);
        if (dbcond != DB_NORMAL) {
	    errmsg("findSCP: Database: DB_startFindRequest Failed (%s):",
		DU_cfindStatusString(dbStatus.status));
        }
    }
    
    /* only cancel if we have pending responses */
    if (cancelled && DICOM_PENDING_STATUS(dbStatus.status)) {
	DB_cancelFindRequest(context->dbHandle, &dbStatus);
    }

    if (DICOM_PENDING_STATUS(dbStatus.status)) {
	dbcond = DB_nextFindResponse(context->dbHandle,
		responseIdentifiers, &dbStatus);
	if (dbcond != DB_NORMAL) {
	     errmsg("findSCP: Database: DB_nextFindResponse Failed (%s):",
		 DU_cfindStatusString(dbStatus.status));
	}
    }

    if (*responseIdentifiers != NULL) {
	addRetreiveAETitle(*responseIdentifiers, context->ourAETitle);
    }

    if (verbose) {
        printf("Find SCP Response %d [status: %s]\n", responseCount,
	    DU_cfindStatusString(dbStatus.status));
    }

    /* set response status */
    response->Status = dbStatus.status;
    *statusDetail = dbStatus.statusDetail;
}


CONDITION
SCE_findSCP(T_ASC_Association * assoc, T_DIMSE_C_FindRQ * request,
	T_ASC_PresentationContextID presID,
	DB_Handle *dbHandle)

{
    CONDITION cond;
    SCE_FindContext context;

    context.dbHandle = dbHandle;
    context.priorStatus = STATUS_Pending;
    ASC_getAPTitles(assoc->params, NULL, context.ourAETitle, NULL);

    if (verbose) {
	printf("Received Find SCP: ");
	DIMSE_printCFindRQ(stdout, request);
    }

    cond = DIMSE_findProvider(assoc, presID, request, 
    	findCallback, &context, DIMSE_BLOCKING, 0);
    if (!SUCCESS(cond)) {
        errmsg("Find SCP Failed:");
	COND_DumpConditions();
    }
    return cond; 
}


/*
** CVS Log
** scefind.cc,v
** Revision 1.3  1996/09/27 08:46:21  hewett
** Enclosed system include files with BEGIN_EXTERN_C/END_EXTERN_C.
**
** Revision 1.2  1996/04/22 10:21:38  hewett
** Formatting change.
**
** Revision 1.1.1.1  1996/03/28 19:24:59  hewett
** Oldenburg Image CTN Software ported to use the dcmdata C++ toolkit.
**
**
*/
