/*
**  Copyright (C) 1993/1994, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/
/*
** 
** Author: Didier Lemoine		Created: 20-07-93 
** 
** Module: db_find 
** 
** Purpose: 	
** This file contains the routines which provide find facilities
** for the DB Module
** 
** Module Prefix: DB_ 
** 
** Last Update:		hewett
** Update Date:		1996/09/27 08:46:49
** Source File:		/share/dicom/cvs-depot/dcmtk/imagectn/libsrc/dbfind.cc,v
** CVS/RCS Revision:	1.5
** Status:		Exp
**
** CVS/RCS Log at end of file
*/

#include "osconfig.h"    /* make sure OS specific configuration is included first */

BEGIN_EXTERN_C
#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif
#include <string.h>
END_EXTERN_C

#include "imagedb.h"
#include "dbpriv.h"
#include "diutil.h"
#include "dcompat.h"
#include "dimse.h"


/************
**	Try to match Two Dates
**	The first one is the "model", the second one an element
**	Returns TRUE if matching is ok, else returns FALSE
 */

int
DB_MatchDate (DB_SmallDcmElmt *mod, DB_SmallDcmElmt *elt)
{
    char date [DBC_MAXSTRING] ;
    char modl [DBC_MAXSTRING] ;

    /*** Get elt and model data in strings
    **/

    memcpy (date, elt -> PValueField, elt -> ValueLength) ;
    date [elt -> ValueLength] = '\0' ;
    DB_RemoveSpaces (date) ;

    memcpy (modl, mod -> PValueField, mod -> ValueLength) ;
    modl [mod -> ValueLength] = '\0' ;
    DB_RemoveSpaces (modl) ;

    /*** If no '-' in date
    *** return strict comparison result
    **/

    if (strchr (modl, '-') == NULL)
	return (strcmp (modl, date) == 0) ;

    /*** If first car is -
    **/

    if (modl [0] == '-') {
	return DB_DateToLong (date) <= DB_DateToLong (modl+1) ;
    }

    /*** If last car is -
    **/

    else if (modl [strlen (modl) - 1] == '-') {
	modl [strlen (modl) - 1] = '\0' ;
	return DB_DateToLong (date) >= DB_DateToLong (modl) ;
    }
    else {
	char *pc ;
	long d ;

	d = DB_DateToLong (date) ;
	pc = strchr (modl, '-') ;
	*pc = '\0' ;

	return (d >= DB_DateToLong (modl)) && (d <= DB_DateToLong (pc+1)) ;

    }
}

/************
**	Try to match Two Times
**	The first one is the "model", the second one an element
**	Returns TRUE if matching is ok, else returns FALSE
 */

int
DB_MatchTime (DB_SmallDcmElmt *mod, DB_SmallDcmElmt *elt)
{
    char time [DBC_MAXSTRING] ;
    char modl [DBC_MAXSTRING] ;

    /*** Get elt and model data in strings
    **/

    memcpy (time, elt -> PValueField, elt -> ValueLength) ;
    time [elt -> ValueLength] = '\0' ;
    DB_RemoveSpaces (time) ;

    memcpy (modl, mod -> PValueField, mod -> ValueLength) ;
    modl [mod -> ValueLength] = '\0' ;
    DB_RemoveSpaces (modl) ;

    /*** If no '-' in time
    *** return strict comparison result
    **/

    if (strchr (modl, '-') == NULL)
	return (strcmp (modl, time) == 0) ;

    /*** If first car is -
    **/

    if (modl [0] == '-') {
	return DB_TimeToDouble (time) <= DB_TimeToDouble (modl+1) ;
    }

    /*** If last car is -
    **/

    else if (modl [strlen (modl) - 1] == '-') {
	modl [strlen (modl) - 1] = '\0' ;
	return DB_TimeToDouble (time) >= DB_TimeToDouble (modl) ;
    }
    else {
	char *pc ;
	double t ;

	t = DB_TimeToDouble (time) ;
	pc = strchr (modl, '-') ;
	*pc = '\0' ;

	return (t >= DB_TimeToDouble (modl)) && (t <= DB_TimeToDouble (pc+1)) ;

    }
}

/************
**	Try to match Two UID
**	The first one is the "model", the second one an element
**	Returns TRUE if matching is ok, else returns FALSE
 */

int
DB_MatchUID (DB_SmallDcmElmt *mod, DB_SmallDcmElmt *elt)
{
    int match ;
    char *uid  ;
    char *modl  ;
    char *pc ;

    /*** Get elt and model data in strings
    **/

    uid = (char *) malloc (elt -> ValueLength + 1) ;
    if (uid == NULL) {
	return 0 ;
    }
    memcpy (uid, elt -> PValueField, elt -> ValueLength) ;
    uid [elt -> ValueLength] = '\0' ;

    modl = (char *) malloc (mod -> ValueLength + 1) ;
    if (modl == NULL) {
	free (uid) ;
	return 0 ;
    }
    memcpy (modl, mod -> PValueField, mod -> ValueLength) ;
    modl [mod -> ValueLength] = '\0' ;


    /*** If no '\' in model
    *** return strict comparison result
    **/

#ifdef STRICT_COMPARE
#else
    /*** Suppress Leading and Trailing spaces in
    *** model and string
    **/

    DB_RemoveEnclosingSpaces (uid) ;
    DB_RemoveEnclosingSpaces (modl) ;
#endif

    if (strchr (modl, '\\') == NULL) {
	match = (strcmp (modl, uid) == 0) ;
	free (uid) ;
	free (modl) ;
	return (match) ;
    }

    /*** UID List comparaison.
    *** Match is successful if uid is found in model
    **/

    match = FALSE ; ;
    for (pc = modl ; *pc ; ) {
	if (strncmp (pc, uid, strlen (uid)) == 0) {
	    match = TRUE ;
	    break ;
	}
	else {
	    pc = strchr (pc, '\\') ;
	    if (pc == NULL)
		break ;
	    else
		pc++ ;
	}
    }

    free (uid) ;
    free (modl) ;
    return (match) ;

}

/************
**	Try to match Two Strings
**	The first one is the "model", the second one an element
**	Returns TRUE if matching is ok, else returns FALSE
 */

int
DB_MatchStrings (DB_SmallDcmElmt *mod, DB_SmallDcmElmt *elt)
{
    int match ;
    char *string  ;
    char *modl  ;

    /*** Get elt and model data in strings
    **/

    string = (char *) malloc (elt -> ValueLength + 1) ;
    if (string == NULL) {
	return 0 ;
    }
    memcpy (string, elt -> PValueField, elt -> ValueLength) ;
    string [elt -> ValueLength] = '\0' ;

    modl = (char *) malloc (mod -> ValueLength + 1) ;
    if (modl == NULL) {
	free (string) ;
	return 0 ;
    }
    memcpy (modl, mod -> PValueField, mod -> ValueLength) ;
    modl [mod -> ValueLength] = '\0' ;

#ifdef STRICT_COMPARE
#else
    /*** Suppress Leading and Trailing spaces in
    *** model and string
    **/

    DB_RemoveEnclosingSpaces (string) ;
    DB_RemoveEnclosingSpaces (modl) ;
#endif

    /*** If no '*' and no '?' in model
    *** return strict comparison result
    **/

    if ((strchr (modl, '*') == NULL) && (strchr (modl, '?') == NULL))
	return (strcmp (modl, string) == 0) ;

    match = DB_StringUnify (modl, string) ;

    free (string) ;
    free (modl) ;
    return (match) ;

}

/************
**	Try to match Two Unknown elements
**	Strict comparaison is applied
**	The first one is the "model", the second one an element
**	Returns TRUE if matching is ok, else returns FALSE
 */

int
DB_MatchOther (DB_SmallDcmElmt *mod, DB_SmallDcmElmt *elt)
{
    if (mod -> ValueLength != elt -> ValueLength)
	return FALSE ;

    return (memcmp (mod -> PValueField, elt -> PValueField, elt -> ValueLength) == 0) ;
}

/************
**	Try to match Two DB_SmallDcmElmts
**	The first one is the "model", the second one an element
**	Returns TRUE if matching is ok, else returns FALSE
 */

int
DB_Match (DB_SmallDcmElmt *mod, DB_SmallDcmElmt *elt)
{
    DB_KEY_CLASS keyClass ;

    /*** If model length is 0
    *** Universal matching is applied : return always TRUE
    **/

    if (mod -> ValueLength == 0)
	return (TRUE) ;

    /*** Get the key class of the element
    **/

    DB_GetTagKeyClass (elt -> XTag, &keyClass) ;

    switch (keyClass) {

    case DATE_CLASS :
	return DB_MatchDate (mod, elt)  ;

    case TIME_CLASS :
	return DB_MatchTime (mod, elt)  ;

    case UID_CLASS :
	return DB_MatchUID  (mod, elt) ;

    case STRING_CLASS :
	return DB_MatchStrings (mod, elt) ;

    case OTHER_CLASS :
	return DB_MatchOther (mod, elt) ;

    }
    return FALSE;
}

/************
**	Create the response list in specified handle,
**	using informations found in an index record.
**	Old response list is supposed freed
**/

void
DB_MakeResponseList (
		DB_Private_Handle	*phandle,
		IdxRecord 		*idxRec
		)
{
    int i ;
    DB_ElementList *pRequestList = NULL;
    DB_ElementList *plist = NULL;
    DB_ElementList *last = NULL;

    phandle -> findResponseList = NULL ;

    /*** For each element in Request identifier
    **/

    for (pRequestList = phandle -> findRequestList ; pRequestList ; pRequestList = pRequestList -> next) {

	/*** Find Corresponding Tag in index record
	**/

	for (i = 0 ; i < NBPARAMETERS ; i++)
	    if (idxRec -> param [i]. XTag == pRequestList -> elem. XTag)
		break ;

	/*** If Tag not found, skip the element
	**/

	if (i >= NBPARAMETERS)
	    continue ;

	/*** Append index record element to response list
	**/

	plist = (DB_ElementList *) malloc (sizeof (DB_ElementList)) ;
	if (plist == NULL) {
	    fprintf(stderr, "DB_MakeResponseList: out of memory\n");
	    return;
	}
	plist -> next = NULL ;

	DB_DuplicateElement(&idxRec->param[i], &plist->elem);

	if (phandle -> findResponseList == NULL) {
	    phandle -> findResponseList = last = plist ;
	}
	else {
	    last -> next = plist ;
	    last = plist ;
	}

    }
}

/***********
** 
*/

static char*
DB_strdup(const char* str)
{
    if (str == NULL) return NULL;
    char* s = (char*)malloc(strlen(str)+1);
    strcpy(s, str);
    return s;
}

/************
**	Add UID in Index Record to the UID found list
 */

void
DB_UIDAddFound (
		DB_Private_Handle	*phandle,
		IdxRecord 		*idxRec
		)
{
    DB_UidList *plist ;

    plist = (DB_UidList *) malloc (sizeof (DB_UidList)) ;
    if (plist == NULL) {
	fprintf(stderr, "DB_UIDAddFound: out of memory\n");
	return;
    }
    plist -> next = phandle -> uidList ;
    plist -> patient = NULL ;
    plist -> study = NULL ;
    plist -> serie = NULL ;
    plist -> image = NULL ;

    if (phandle -> queryLevel >= PATIENT_LEVEL)
	plist -> patient = DB_strdup ((char *) idxRec -> PatientID) ;
    if (phandle -> queryLevel >= STUDY_LEVEL)
	plist -> study = DB_strdup ((char *) idxRec -> StudyInstanceUID) ;
    if (phandle -> queryLevel >= SERIE_LEVEL)
	plist -> serie = DB_strdup ((char *) idxRec -> SeriesInstanceUID) ;
    if (phandle -> queryLevel >= IMAGE_LEVEL)
	plist -> image = DB_strdup ((char *) idxRec -> SOPInstanceUID) ;

    phandle -> uidList = plist ;
}


/************
**	Search if an Index Record has already been found
 */

int
DB_UIDAlreadyFound (
		DB_Private_Handle	*phandle,
		IdxRecord 		*idxRec
		)
{
    DB_UidList *plist ;

    for (plist = phandle -> uidList ; plist ; plist = plist -> next) {
	if (  (phandle -> queryLevel >= PATIENT_LEVEL)
	      && (strcmp (plist -> patient, (char *) idxRec -> PatientID) != 0)
	    )
	    continue ;
	if (  (phandle -> queryLevel >= STUDY_LEVEL)
	      && (strcmp (plist -> study, (char *) idxRec -> StudyInstanceUID) != 0)
	    )
	    continue ;
	if (  (phandle -> queryLevel >= SERIE_LEVEL)
	      && (strcmp (plist -> serie, (char *) idxRec -> SeriesInstanceUID) != 0)
	    )
	    continue ;
	if (  (phandle -> queryLevel >= IMAGE_LEVEL)
	      && (strcmp (plist -> image, (char *) idxRec -> SOPInstanceUID) != 0)
	    )
	    continue ;
	return (TRUE) ;
    }
    return (FALSE) ;
}

/************
**	Test a Find Request List
**	Returns DB_NORMAL if ok, else returns DB_ERROR
 */

CONDITION
DB_TestFindRequestList (
		DB_ElementList 	*findRequestList,
		DB_LEVEL	queryLevel,
		DB_LEVEL	infLevel
		)
{
    DB_ElementList *plist ;
    DB_LEVEL 	XTagLevel ;
    DB_KEY_TYPE 	XTagType ;
    int	level ;

    /**** Query level must be at least the infLevel
    ***/

    if (queryLevel < infLevel) {
	DB_debug(1,"Level incompatible with Information Model (level %d)\n", queryLevel) ;
	return DB_ERROR ;
    }

    for (level = PATIENT_LEVEL ; level <= IMAGE_LEVEL ; level++) {

	/**** Manage exception due to StudyRoot Information Model :
	**** In this information model, queries may include Patient attributes
	**** but only if they are made at the study level
	***/

	if ((level == PATIENT_LEVEL) && (infLevel == STUDY_LEVEL)) {
	    /** In Study Root Information Model, accept only Patient Tags
	    ** if the Query Level is the Study level
	    */

	    int atLeastOneKeyFound = FALSE ;
	    for (plist = findRequestList ; plist ; plist = plist -> next) {
		DB_GetTagLevel (plist -> elem. XTag, &XTagLevel) ;
		if (XTagLevel != level)
		    continue ;
		atLeastOneKeyFound = TRUE ;
	    }
	    if (atLeastOneKeyFound && (queryLevel != STUDY_LEVEL)) {
		DB_debug(1,"Key found in Study Root Information Model (level %d)\n", level) ;
		return DB_ERROR ;
	    }
	}

	/**** If current level is above the QueryLevel
	***/

	else if (level < queryLevel) {

	    /** For this level, only unique keys are allowed
	    ** Parse the request list elements reffering to
	    ** this level.
	    ** Check that only unique key attr are provided
	    */

	    int uniqueKeyFound = FALSE ;
	    for (plist = findRequestList ; plist ; plist = plist -> next) {
		DB_GetTagLevel (plist -> elem. XTag, &XTagLevel) ;
		if (XTagLevel != level)
		    continue ;
		DB_GetTagKeyAttr (plist -> elem. XTag, &XTagType) ;
		if (XTagType != UNIQUE) {
		    DB_debug(1,"Non Unique Key found (level %d)\n", level) ;
		    return DB_ERROR ;
		}
		else if (uniqueKeyFound) {
		    DB_debug(1,"More than one Unique Key found (level %d)\n", level) ;
		    return DB_ERROR ;
		}
		else
		    uniqueKeyFound = TRUE ;
	    }
	}

	/**** If current level is the QueryLevel
	***/

	else if (level == queryLevel) {

	    /** For this level, all keys are allowed
	    ** Parse the request list elements reffering to
	    ** this level.
	    ** Check that at least one key is provided
	    */

	    int atLeastOneKeyFound = FALSE ;
	    for (plist = findRequestList ; plist ; plist = plist -> next) {
		DB_GetTagLevel (plist -> elem. XTag, &XTagLevel) ;
		if (XTagLevel != level)
		    continue ;
		atLeastOneKeyFound = TRUE ;
	    }
	    if (! atLeastOneKeyFound) {
		DB_debug(1,"No Key found at query level (level %d)\n", level) ;
		return DB_ERROR ;
	    }
	}

	/**** If current level beyond the QueryLevel
	***/

	else if (level > queryLevel) {

	    /** For this level, no key is allowed
	    ** Parse the request list elements reffering to
	    ** this level.
	    ** Check that no key is provided
	    */

	    int atLeastOneKeyFound = FALSE ;
	    for (plist = findRequestList ; plist ; plist = plist -> next) {
		DB_GetTagLevel (plist -> elem. XTag, &XTagLevel) ;
		if (XTagLevel != level)
		    continue ;
		atLeastOneKeyFound = TRUE ;
	    }
	    if (atLeastOneKeyFound) {
		DB_debug(1,"Key found beyond query level (level %d)\n", level) ;
		return DB_ERROR ;
	    }
	}

    }
    return DB_NORMAL ;
}


/************
**	Hierarchical Search Algorithm
**	Returns TRUE if matching is ok, else returns FALSE
 */

CONDITION
DB_HierarchicalCompare (
		DB_Private_Handle	*phandle,
		IdxRecord 		*idxRec,
		DB_LEVEL		level,
		DB_LEVEL		infLevel,
		int			*match
		)
{
    int 		i ;
    DcmTagKey 	XTag ;
    DB_ElementList *plist ;
    DB_LEVEL 	XTagLevel ;

    /**** If current level is above the QueryLevel
    ***/

    if (level < phandle -> queryLevel) {

	/** Get UID Tag for current level
	 */

	DB_GetUIDTag (level, &XTag) ;

	/** Find Element with this XTag in Identifier list
	 */

	for (plist = phandle -> findRequestList ; plist ; plist = plist -> next)
	    if (plist -> elem. XTag == XTag)
		break ;

	/** Element not found
	 */

	if (plist == NULL) {
	    *match = FALSE ;
	    fprintf(stderr, "DB_HierarchicalCompare : No UID Key found at level %d\n", level) ;

	    return DB_ERROR ;
	}

	/** Find element with the same XTag in index record
	 */

	for (i = 0 ; i < NBPARAMETERS ; i++)
	    if (idxRec -> param [i]. XTag == XTag)
		break ;

	/** Compare with Single value matching
	** If Match fails, return FALSE
	*/

	if (! DB_Match (&(plist -> elem), &idxRec->param[i])) {
	    *match = FALSE ;
	    return DB_NORMAL ;
	}

	/** Match succeeded.
	** Try at next level
	*/

	return DB_HierarchicalCompare (phandle, idxRec, (DB_LEVEL)(level + 1), infLevel, match) ;
    }

    /**** If current level is the QueryLevel
    ***/

    else if (level == phandle -> queryLevel) {

	/*** For each element in Identifier list
	**/

	for (plist = phandle -> findRequestList ; plist ; plist = plist -> next) {

	    /** Get the Tag level of this element
	     */

	    DB_GetTagLevel (plist -> elem. XTag, &XTagLevel) ;

	    /** If we are in the Study Root Information Model exception
	    ** we must accept patients keys at the study level
	    */
 
	    if (  (XTagLevel == PATIENT_LEVEL)
		  && (phandle -> queryLevel == STUDY_LEVEL)
		  && (infLevel == STUDY_LEVEL)
		) ;

	    /** In other cases, only keys at the current level are
	    ** taken into account. So skip this element.
	    */

	    else if (XTagLevel != level) 
		continue ;

	    /** Find element with the same XTag in index record
	     */

	    for (i = 0 ; i < NBPARAMETERS ; i++)
		if (idxRec -> param [i]. XTag == plist -> elem. XTag)
		    break ;

	    /** Compare with appropriate Matching.
	    ** If Match fails, return FALSE
	    */


	    if (! DB_Match (&(plist -> elem), &idxRec->param[i])) {
		*match = FALSE ;
		return DB_NORMAL ;
	    }
	}

	/*** If we are here, all matches succeeded at the current level.
	*** Perhaps check that we have tried at least one match ??
	**/

	*match = TRUE ;
	return DB_NORMAL ;

    }
    return DB_ERROR;
}

/********************
**	Start find in Database
**/

CONDITION 
DB_startFindRequest(
		DB_Handle 	*handle, 			
		const char 	*SOPClassUID,
		DcmDataset 	*findRequestIdentifiers,
		DB_Status 	*status)
{
    DB_Private_Handle	*phandle = NULL;
    DB_SmallDcmElmt 	elem ;
    DB_ElementList 	*plist = NULL;
    DB_ElementList 	*last = NULL;
    int 		MatchFound ;
    IdxRecord 		idxRec ;
    DB_LEVEL		qLevel = PATIENT_LEVEL;
    CONDITION		cond = DB_NORMAL;
    BOOLEAN qrLevelFound = FALSE;

    phandle = (DB_Private_Handle *) handle ; 

    /**** Is SOPClassUID supported ?
    ***/

    if (strcmp( SOPClassUID, UID_FINDPatientRootQueryRetrieveInformationModel) == 0)
	phandle -> rootLevel = PATIENT_ROOT ;
    else if (strcmp( SOPClassUID, UID_FINDStudyRootQueryRetrieveInformationModel) == 0)
	phandle -> rootLevel = STUDY_ROOT ;
#ifdef PATIENTSTUDYONLY_IS_SUPPORTED
    else if (strcmp( SOPClassUID, UID_FINDPatientStudyOnlyQueryRetrieveInformationModel) == 0)
        phandle -> rootLevel = PATIENT_STUDY ;
#endif
    else {
	status -> status = STATUS_FIND_Refused_SOPClassNotSupported ;
	return (DB_ERROR) ;
    }


    /**** Parse Identifiers in the Dicom Object
    **** Find Query Level and contruct a list
    **** of query identifiers
    ***/
	
    phandle -> findRequestList = NULL ;

    int elemCount = findRequestIdentifiers->card();
    for (int elemIndex=0; elemIndex<elemCount; elemIndex++) {

	DcmElement* dcelem = findRequestIdentifiers->getElement(elemIndex);

	elem.XTag = dcelem->getTag().getXTag();
	if (elem.XTag == DCM_QueryRetrieveLevel || DB_TagSupported(elem.XTag)) {
	    elem.ValueLength = dcelem->getLength();
	    if (elem.ValueLength == 0) {
		elem.PValueField = NULL ;
	    } else if ((elem.PValueField = (char*)malloc(elem.ValueLength+1)) == NULL) {
		status -> status = STATUS_FIND_Refused_OutOfResources ;
		return (DB_ERROR) ;
	    } else {
		/* only char string type tags are supported at the moment */
		char *s = NULL;
		dcelem->get(s);
		strcpy(elem.PValueField, s);
	    }
	    /** If element is the Query Level, store it in handle
	     */

	    if (elem. XTag == DCM_QueryRetrieveLevel) {
		char *pc ;
		char level [50] ;

		strncpy(level, (char*)elem.PValueField, 
			(elem.ValueLength<50)? elem.ValueLength : 49) ;

		/*** Skip this two lines if you want strict comparison
		**/

		for (pc = level ; *pc ; pc++)
		    *pc = ((*pc >= 'a') && (*pc <= 'z')) ? 'A' - 'a' + *pc : *pc ;

		if (strncmp (level, PATIENT_LEVEL_STRING, 
			     strlen (PATIENT_LEVEL_STRING)) == 0)
		    phandle -> queryLevel = PATIENT_LEVEL ;
		else if (strncmp (level, STUDY_LEVEL_STRING, 
				  strlen (STUDY_LEVEL_STRING)) == 0)
		    phandle -> queryLevel = STUDY_LEVEL ;
		else if (strncmp (level, SERIE_LEVEL_STRING, 
				  strlen (SERIE_LEVEL_STRING)) == 0)
		    phandle -> queryLevel = SERIE_LEVEL ;
		else if (strncmp (level, IMAGE_LEVEL_STRING, 
				  strlen (IMAGE_LEVEL_STRING)) == 0)
		    phandle -> queryLevel = IMAGE_LEVEL ;
		else {
		    if (elem. PValueField)
			free (elem. PValueField) ;
#ifdef DEBUG
		    DB_debug(1, "DB_startFindRequest () : Illegal query level (%s)\n", level) ;
#endif
		    status -> status = STATUS_FIND_Failed_UnableToProcess ;
		    return (DB_ERROR) ;
		}
		qrLevelFound = TRUE;
	    } else {
		/** Else it is a query identifier.
		** Append it to our RequestList if it is supported
		*/
		if (DB_TagSupported (elem. XTag)) {
			
		    plist = (DB_ElementList *) malloc (sizeof (DB_ElementList)) ;
		    if (plist == NULL) {
			status -> status = STATUS_FIND_Refused_OutOfResources ;
			return (DB_ERROR) ;
		    }
		    plist -> next = NULL ;
		    DB_DuplicateElement (&elem, &(plist -> elem)) ;
		    if (phandle -> findRequestList == NULL) {
			phandle -> findRequestList = last = plist ;
		    } else {
			last -> next = plist ;
			last = plist ;
		    }
		}
	    }

	    if ( elem. PValueField ) {
		free (elem. PValueField) ;
	    }
	}
    }

    if (!qrLevelFound) {
	/* The Query/Retrieve Level is missing */
	status->status = STATUS_FIND_Failed_IdentifierDoesNotMatchSOPClass ;
	fprintf(stderr, 
		"DB_startFindRequest(): missing Query/Retrieve Level\n");
	phandle -> idxCounter = -1 ;
	DB_FreeElementList (phandle -> findRequestList) ;
	phandle -> findRequestList = NULL ;
	return (DB_ERROR) ;
    }

    switch (phandle -> rootLevel) {
    case PATIENT_ROOT :	qLevel = PATIENT_LEVEL ;	break ;
    case STUDY_ROOT :	qLevel = STUDY_LEVEL ;		break ;
    case PATIENT_STUDY: qLevel = PATIENT_LEVEL ;	break ;
    }

    /**** Test the consistency of the request list
    ***/

    if (DB_doCheckFindIdentifier()) {
	cond = DB_TestFindRequestList (phandle -> findRequestList, phandle -> queryLevel, qLevel) ;
	if (cond != DB_NORMAL) {
	    phandle -> idxCounter = -1 ;
	    DB_FreeElementList (phandle -> findRequestList) ;
	    phandle -> findRequestList = NULL ;
#ifdef DEBUG
	    DB_debug(1, "DB_startFindRequest () : STATUS_FIND_Failed_IdentifierDoesNotMatchSOPClass - Invalid RequestList\n") ;
#endif
	    status -> status = STATUS_FIND_Failed_IdentifierDoesNotMatchSOPClass ;
	    return (cond) ;
	}
    }

    /**** Goto the beginning of Index File
    **** Then find the first matching image
    ***/

    DB_lock(phandle, FALSE);

    DB_IdxInitLoop (phandle, &(phandle -> idxCounter)) ;
    MatchFound = FALSE ;
    cond = DB_NORMAL ;

    while (1) {

	/*** Exit loop if read error (or end of file)
	**/

	if (DB_IdxGetNext (phandle, &(phandle -> idxCounter), &idxRec) != DB_NORMAL)
	    break ;

	/*** Exit loop if error or matching OK
	**/

	cond = DB_HierarchicalCompare (phandle, &idxRec, qLevel, qLevel, &MatchFound) ;
	if (cond != DB_NORMAL)
	    break ;
	if (MatchFound)
	    break ;
    }

    /**** If an error occured in Matching function
    ****    return a failed status
    ***/

    if (cond != DB_NORMAL) {
	phandle -> idxCounter = -1 ;
	DB_FreeElementList (phandle -> findRequestList) ;
	phandle -> findRequestList = NULL ;
#ifdef DEBUG
	DB_debug(1, "DB_startFindRequest () : STATUS_FIND_Failed_UnableToProcess\n") ;
#endif
	status -> status = STATUS_FIND_Failed_UnableToProcess ;

	DB_unlock(phandle);

	return (cond) ;
    }


    /**** If a matching image has been found,
    ****	 add index record to UID found list
    ****    prepare Response List in handle
    ****    return status is pending
    ***/

    if (MatchFound) {
	DB_UIDAddFound (phandle, &idxRec) ;
	DB_MakeResponseList (phandle, &idxRec) ;
#ifdef DEBUG
	DB_debug(1, "DB_startFindRequest () : STATUS_Pending\n") ;
#endif
	status -> status = STATUS_Pending ;
	return (DB_NORMAL) ;
    }

    /**** else no matching image has been found,
    ****    free query identifiers list
    ****    status is success
    ***/

    else {
	phandle -> idxCounter = -1 ;
	DB_FreeElementList (phandle -> findRequestList) ;
	phandle -> findRequestList = NULL ;
#ifdef DEBUG
	DB_debug(1, "DB_startFindRequest () : STATUS_Success\n") ;
#endif
	status -> status = STATUS_Success ;

	DB_unlock(phandle);

	return (DB_NORMAL) ;
    }

}

/********************
**	Get next find response in Database
 */

CONDITION 
DB_nextFindResponse (
		DB_Handle 	*handle, 			
		DcmDataset 	**findResponseIdentifiers,
		DB_Status 	*status)
{

    DB_Private_Handle	*phandle = NULL; 
    DB_ElementList 	*plist = NULL;
    int 		MatchFound = FALSE;
    IdxRecord 		idxRec ;
    DB_LEVEL 		qLevel = PATIENT_LEVEL;
    char		*queryLevelString = NULL;
    CONDITION		cond = DB_NORMAL;

    phandle = (DB_Private_Handle *) handle ; 


    if (phandle -> findResponseList == NULL) {
#ifdef DEBUG
	DB_debug(1, "DB_nextFindResponse () : STATUS_Success\n") ;
#endif
	*findResponseIdentifiers = NULL ;
	status -> status = STATUS_Success ;

	DB_unlock(phandle);

	return (DB_NORMAL) ;
    }	

    /***** Create the response (findResponseIdentifiers) using
    ***** the last find done and saved in handle findResponseList
    ****/

    *findResponseIdentifiers = new DcmDataset ;
    if ( *findResponseIdentifiers != NULL ) {

	/*** Put responses
	**/

	for ( plist = phandle -> findResponseList ; plist != NULL ; plist = plist -> next ) {
	    DcmTag t(plist->elem.XTag);
	    DcmElement *dce = newDicomElement(t);
	    if (dce == NULL) {
		status -> status = STATUS_FIND_Refused_OutOfResources ;
		return DB_ERROR;
	    }
	    if (plist->elem.PValueField != NULL && 
		strlen(plist->elem.PValueField) > 0) {
		E_Condition ec = dce->put(plist->elem.PValueField);
		if (ec != EC_Normal) {
		    fprintf(stderr, "dbfind: DB_nextFindResponse: cannot put()\n");
		    status -> status = STATUS_FIND_Failed_UnableToProcess ;
		    return DB_ERROR;
		}
	    }
	    E_Condition ec = (*findResponseIdentifiers)->insert(dce);
	    if (ec != EC_Normal) {
		fprintf(stderr, "dbfind: DB_nextFindResponse: cannot insert()\n");
		status -> status = STATUS_FIND_Failed_UnableToProcess ;
		return DB_ERROR;
	    }
	}

	/*** Append the Query level
	**/

	switch (phandle -> queryLevel) {
	case PATIENT_LEVEL :
	    queryLevelString = PATIENT_LEVEL_STRING ;
	    break ;
	case STUDY_LEVEL :
	    queryLevelString = STUDY_LEVEL_STRING ;
	    break ;
	case SERIE_LEVEL :
	    queryLevelString = SERIE_LEVEL_STRING ;
	    break ;
	case IMAGE_LEVEL :
	    queryLevelString = IMAGE_LEVEL_STRING ;
	    break ;
	}
	DU_putStringDOElement(*findResponseIdentifiers,
			      DCM_QueryRetrieveLevel, queryLevelString);
#ifdef DEBUG
	if (DB_debugLevel > 0) {
	    printf("DB: findResponseIdentifiers:\n");
	    (*findResponseIdentifiers)->print();
	}
#endif
    }
    else {

	DB_unlock(phandle);

	return (DB_ERROR) ;
    }
	
    switch (phandle -> rootLevel) {
    case PATIENT_ROOT :	qLevel = PATIENT_LEVEL ;	break ;
    case STUDY_ROOT :	qLevel = STUDY_LEVEL ;		break ;
    case PATIENT_STUDY: qLevel = PATIENT_LEVEL ;        break ; 
    }

    /***** Free the last response...
    ****/

    DB_FreeElementList (phandle -> findResponseList) ;
    phandle -> findResponseList = NULL ;

    /***** ... and find the next one
    ****/

    MatchFound = FALSE ;
    cond = DB_NORMAL ;

    while (1) {

	/*** Exit loop if read error (or end of file)
	**/

	if (DB_IdxGetNext (phandle, &(phandle -> idxCounter), &idxRec) != DB_NORMAL)
	    break ;

	/*** If Response already found
	**/

	if (DB_UIDAlreadyFound (phandle, &idxRec))
	    continue ;

	/*** Exit loop if error or matching OK
	**/

	cond = DB_HierarchicalCompare (phandle, &idxRec, qLevel, qLevel, &MatchFound) ;
	if (cond != DB_NORMAL)
	    break ;
	if (MatchFound)
	    break ;

    }

    /**** If an error occured in Matching function
    ****    return status is pending
    ***/

    if (cond != DB_NORMAL) {
	phandle -> idxCounter = -1 ;
	DB_FreeElementList (phandle -> findRequestList) ;
	phandle -> findRequestList = NULL ;
#ifdef DEBUG
	DB_debug(1, "DB_nextFindResponse () : STATUS_FIND_Failed_UnableToProcess\n") ;
#endif
	status -> status = STATUS_FIND_Failed_UnableToProcess ;

	DB_unlock(phandle);

	return (cond) ;
    }


    /**** If a matching image has been found
    ****    add index records UIDs in found UID list
    ****    prepare Response List in handle
    ***/

    if (MatchFound) {
	DB_UIDAddFound (phandle, &idxRec) ;
	DB_MakeResponseList (phandle, &idxRec) ;
#ifdef DEBUG
	DB_debug(1, "DB_nextFindResponse () : STATUS_Pending\n") ;
#endif
	status -> status = STATUS_Pending ;
	return (DB_NORMAL) ;
    }

    /**** else no matching image has been found,
    ****    free query identifiers list
    **** Response list is null, so next call will return STATUS_Success
    ***/

    else {
	phandle -> idxCounter = -1 ;
	DB_FreeElementList (phandle -> findRequestList) ;
	phandle -> findRequestList = NULL ;
	DB_FreeUidList (phandle -> uidList) ;
	phandle -> uidList = NULL ;
    }


#ifdef DEBUG
    DB_debug(1, "DB_nextFindResponse () : STATUS_Pending\n") ;
#endif
    status -> status = STATUS_Pending ;
    return (DB_NORMAL) ;
}

/********************
**	Cancel find request
 */

CONDITION 
DB_cancelFindRequest (DB_Handle *handle, DB_Status *status) 
{
    DB_Private_Handle	*phandle ; 


    phandle = (DB_Private_Handle *) handle ; 

    phandle -> idxCounter = -1 ;
    DB_FreeElementList (phandle -> findRequestList) ;
    phandle -> findRequestList = NULL ;
    DB_FreeElementList (phandle -> findResponseList) ;
    phandle -> findResponseList = NULL ;
    DB_FreeUidList (phandle -> uidList) ;
    phandle -> uidList = NULL ;

    status -> status = STATUS_FIND_Cancel_MatchingTerminatedDueToCancelRequest ;

    DB_unlock(phandle);

    return (DB_NORMAL) ;
}

/*
** CVS Log
** dbfind.cc,v
** Revision 1.5  1996/09/27 08:46:49  hewett
** Enclosed system include files with BEGIN_EXTERN_C/END_EXTERN_C.
**
** Revision 1.4  1996/04/27 12:55:17  hewett
** Removed cause of warnings when compiled with "c++ -O -g -Wall" under
** Solaris 2.4.  Mostly due to uninitialized variables.
**
** Revision 1.3  1996/04/25 16:30:23  hewett
** Replaced strdup call with private function DB_strdup().
**
** Revision 1.2  1996/04/22 10:37:48  hewett
** Now only processes supported keys.  Others are ignored.  Added check for
** presence of Query/Retrieve Level.
**
** Revision 1.1.1.1  1996/03/28 19:25:00  hewett
** Oldenburg Image CTN Software ported to use the dcmdata C++ toolkit.
**
**
*/
