/*
**  Copyright (C) 1993/1994, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/
/*
** 
** Author: Didier Lemoine		Created: 20-07-93 
**	   CERIUM, Rennes, France
** 
** Module: db_move 
** 
** Purpose: 	
** This file contains the routines which provide move facilities
** for the DB Module
** 
** Module Prefix: DB_ 
** 
** Module Prefix: DB_ 
** 
** Last Update:		hewett
** Update Date:		1996/09/27 08:46:50
** Source File:		/share/dicom/cvs-depot/dcmtk/imagectn/libsrc/dbmove.cc,v
** CVS/RCS Revision:	1.4
** Status:		Exp
**
** CVS/RCS Log at end of file
*/

#include "osconfig.h"    /* make sure OS specific configuration is included first */

BEGIN_EXTERN_C
#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif
END_EXTERN_C

#include "imagedb.h"
#include "dbpriv.h"
#include "dcdatset.h"
#include "dcdeftag.h"
#include "dimse.h"


/************
 *	Test a Move Request List
 *	Returns DB_NORMAL if ok, else returns DB_ERROR
 */

CONDITION
DB_TestMoveRequestList (
		DB_ElementList 	*findRequestList,
		DB_LEVEL	queryLevel,
		DB_LEVEL	infLevel
		)
{
    DB_ElementList *plist ;
    DB_LEVEL 	XTagLevel ;
    DB_KEY_TYPE 	XTagType ;
    int	level ;

    /**** Query level must be at least the infLevel
    ***/

    if (queryLevel < infLevel) {
	DB_debug(1,"Level incompatible with Information Model (level %d)\n", (int)queryLevel) ;
	return DB_ERROR ;
    }

    for (level = PATIENT_LEVEL ; level <= IMAGE_LEVEL ; level++) {

	/**** Manage exception due to StudyRoot Information Model :
	**** In this information model, move may not include any
	**** Patient attributes.
	***/

	if ((level == PATIENT_LEVEL) && (infLevel == STUDY_LEVEL)) {

	    /** In Study Root Information Model, do not accept any 
	    ** Patient Tag
	    */

	    int atLeastOneKeyFound = FALSE ;
	    for (plist = findRequestList ; plist ; plist = plist -> next) {
		DB_GetTagLevel (plist -> elem. XTag, &XTagLevel) ;
		if (XTagLevel != level)
		    continue ;
		atLeastOneKeyFound = TRUE ;
	    }
	    if (atLeastOneKeyFound) {
		DB_debug(1,"Key found in Study Root Information Model (level %d)\n", level) ;
		return DB_ERROR ;
	    }
	}

	/**** If current level is above or equal to the QueryLevel
	***/

	else if (level <= queryLevel) {

	    /** For these levels, only unique keys are allowed
	    ** Parse the request list elements reffering to
	    ** this level.
	    ** Check that only unique key attr are provided
	    */

	    int uniqueKeyFound = FALSE ;
	    for (plist = findRequestList ; plist ; plist = plist -> next) {
		DB_GetTagLevel (plist -> elem. XTag, &XTagLevel) ;
		if (XTagLevel != level)
		    continue ;
		DB_GetTagKeyAttr (plist -> elem. XTag, &XTagType) ;
		if (XTagType != UNIQUE) {
		    DB_debug(1,"Non Unique Key found (level %d)\n", level) ;
		    return DB_ERROR ;
		}
		else if (uniqueKeyFound) {
		    DB_debug(1,"More than one Unique Key found (level %d)\n", level) ;
		    return DB_ERROR ;
		}
		else
		    uniqueKeyFound = TRUE ;
	    }
	    if (! uniqueKeyFound) {
		DB_debug(1,"No Unique Key found (level %d)\n", level) ;
		return DB_ERROR ;
	    }
	}

	/**** If current level beyond the QueryLevel
	***/

	else if (level > queryLevel) {

	    /** For this level, no key is allowed
	    ** Parse the request list elements reffering to
	    ** this level.
	    ** Check that no key is provided
	    */

	    int atLeastOneKeyFound = FALSE ;
	    for (plist = findRequestList ; plist ; plist = plist -> next) {
		DB_GetTagLevel (plist -> elem. XTag, &XTagLevel) ;
		if (XTagLevel != level)
		    continue ;
		atLeastOneKeyFound = TRUE ;
	    }
	    if (atLeastOneKeyFound) {
		DB_debug(1,"Key found beyond query level (level %d)\n", level) ;
		return DB_ERROR ;
	    }
	}

    }
    return DB_NORMAL ;
}



CONDITION 
DB_startMoveRequest(
	DB_Handle 	*handle, 
	const char 	*SOPClassUID, 
	DcmDataset 	*moveRequestIdentifiers, 
	DB_Status 	*status) 
{

    DB_Private_Handle	*phandle = NULL;
    DB_SmallDcmElmt	elem ;
    DB_ElementList 	*plist = NULL;
    DB_ElementList 	*last = NULL;
    DB_CounterList	*pidxlist = NULL;
    DB_CounterList	*lastidxlist = NULL;
    int 		MatchFound = FALSE;
    IdxRecord 		idxRec ;
    DB_LEVEL 		qLevel = PATIENT_LEVEL;
    CONDITION		cond = DB_NORMAL;
    BOOLEAN qrLevelFound = FALSE;

    phandle = (DB_Private_Handle *) handle ; 

    /**** Is SOPClassUID supported ?
    ***/

    if (strcmp( SOPClassUID, UID_MOVEPatientRootQueryRetrieveInformationModel) == 0)
	phandle -> rootLevel = PATIENT_ROOT ;
    else if (strcmp( SOPClassUID, UID_MOVEStudyRootQueryRetrieveInformationModel) == 0)
	phandle -> rootLevel = STUDY_ROOT ;
#ifdef PATIENTSTUDYONLY_IS_SUPPORTED
    else if (strcmp( SOPClassUID, UID_MOVEPatientStudyOnlyQueryRetrieveInformationModel) == 0)
        phandle -> rootLevel = PATIENT_STUDY ;
#endif
    else {
	status -> status = STATUS_MOVE_Failed_SOPClassNotSupported ;
	return (DB_ERROR) ;
    }


    /**** Parse Identifiers in the Dicom Object
    **** Find Query Level and contruct a list
    **** of query identifiers
    ***/
	
    int elemCount = moveRequestIdentifiers->card();
    for (int elemIndex=0; elemIndex<elemCount; elemIndex++) {

	DcmElement* dcelem = moveRequestIdentifiers->getElement(elemIndex);

	elem.XTag = dcelem->getTag().getXTag();
	if (elem.XTag == DCM_QueryRetrieveLevel || DB_TagSupported(elem.XTag)) {
	    elem.ValueLength = dcelem->getLength();
	    if (elem.ValueLength == 0) {
		elem.PValueField = NULL ;
	    } else if ((elem.PValueField = (char*)malloc(elem.ValueLength+1)) == NULL) {
		status -> status = STATUS_MOVE_Failed_UnableToProcess ;
		return (DB_ERROR) ;
	    } else {
		/* only char string type tags are supported at the moment */
		char *s = NULL;
		dcelem->get(s);
		strcpy(elem.PValueField, s);
	    }

	    /** If element is the Query Level, store it in handle
	     */

	    if (elem. XTag == DCM_QueryRetrieveLevel) {
		char *pc ;
		char level [50] ;

		strncpy (level, (char *) elem. PValueField, (elem. ValueLength < 50) ? elem. ValueLength : 49) ;
		
		/*** Skip this two lines if you want strict comparison
		**/

		for (pc = level ; *pc ; pc++)
		    *pc = ((*pc >= 'a') && (*pc <= 'z')) ? 'A' - 'a' + *pc : *pc ;

		if (strncmp (level, PATIENT_LEVEL_STRING, 
			     strlen (PATIENT_LEVEL_STRING)) == 0)
		    phandle -> queryLevel = PATIENT_LEVEL ;
		else if (strncmp (level, STUDY_LEVEL_STRING, 
				  strlen (STUDY_LEVEL_STRING)) == 0)
		    phandle -> queryLevel = STUDY_LEVEL ;
		else if (strncmp (level, SERIE_LEVEL_STRING, 
				  strlen (SERIE_LEVEL_STRING)) == 0)
		    phandle -> queryLevel = SERIE_LEVEL ;
		else if (strncmp (level, IMAGE_LEVEL_STRING, 
				  strlen (IMAGE_LEVEL_STRING)) == 0)
		    phandle -> queryLevel = IMAGE_LEVEL ;
		else {
#ifdef DEBUG
		    DB_debug(1,"DB_startMoveRequest : STATUS_MOVE_Failed_UnableToProcess\n") ;
#endif
		    status -> status = STATUS_MOVE_Failed_UnableToProcess ;
		    return (DB_ERROR) ;
		}
		qrLevelFound = TRUE;
	    } else {
		/** Else it is a query identifier
		** Append it to our RequestList
		*/
		if (! DB_TagSupported (elem. XTag))
		    continue ;
			
		plist = (DB_ElementList *) malloc (sizeof( DB_ElementList ) ) ;
		if (plist == NULL) {
		    status -> status = STATUS_FIND_Refused_OutOfResources ;
		    return (DB_ERROR) ;
		}
		plist -> next = NULL ;
		DB_DuplicateElement (&elem, & (plist -> elem)) ;
		if (phandle -> findRequestList == NULL) {
		    phandle -> findRequestList = last = plist ;
		} else {
		    last -> next = plist ;
		    last = plist ;
		}
	    }

	    if ( elem. PValueField ) {
		free (elem. PValueField) ;
	    }
	}
    }

    if (!qrLevelFound) {
	/* The Query/Retrieve Level is missing */
	status->status = STATUS_MOVE_Failed_IdentifierDoesNotMatchSOPClass ;
	fprintf(stderr, 
		"DB_startMoveRequest(): missing Query/Retrieve Level\n");
	phandle -> idxCounter = -1 ;
	DB_FreeElementList (phandle -> findRequestList) ;
	phandle -> findRequestList = NULL ;
	return (DB_ERROR) ;
    }

    switch (phandle -> rootLevel) {
    case PATIENT_ROOT :	qLevel = PATIENT_LEVEL ;	break ;
    case STUDY_ROOT :	qLevel = STUDY_LEVEL ;		break ;
    case PATIENT_STUDY: qLevel = PATIENT_LEVEL ;	break ;
    }

    /**** Test the consistency of the request list
    ***/

    if (DB_doCheckMoveIdentifier()) {
	cond = DB_TestMoveRequestList (phandle -> findRequestList, 
				       phandle -> queryLevel, qLevel) ;
	if (cond != DB_NORMAL) {
	    phandle -> idxCounter = -1 ;
	    DB_FreeElementList (phandle -> findRequestList) ;
	    phandle -> findRequestList = NULL ;
#ifdef DEBUG
	    DB_debug(1,"DB_startMoveRequest () : STATUS_MOVE_Failed_IdentifierDoesNotMatchSOPClass - Invalid RequestList\n") ;
#endif
	    status -> status = STATUS_MOVE_Failed_IdentifierDoesNotMatchSOPClass ;
	    return (cond) ;
	}
    }

    /**** Goto the beginning of Index File
    **** Then find all matching images
    ***/

    MatchFound = FALSE ;
    phandle -> moveCounterList = NULL ;
    phandle -> NumberRemainOperations = 0 ;

    /**** Find matching images
    ***/

    DB_lock(phandle, FALSE);

    DB_IdxInitLoop (phandle, &(phandle -> idxCounter)) ;
    while (1) {

	/*** Exit loop if read error (or end of file)
	**/

	if (DB_IdxGetNext (phandle, &(phandle -> idxCounter), &idxRec) != DB_NORMAL)
	    break ;

	/*** If matching found
	**/

	cond = DB_HierarchicalCompare (phandle, &idxRec, qLevel, qLevel, &MatchFound) ;
	if (MatchFound) {
	    pidxlist = (DB_CounterList *) malloc (sizeof( DB_CounterList ) ) ;
	    if (pidxlist == NULL) {
		status -> status = STATUS_FIND_Refused_OutOfResources ;
		return (DB_ERROR) ;
	    }

	    pidxlist -> next = NULL ;
	    pidxlist -> idxCounter = phandle -> idxCounter ;
	    phandle -> NumberRemainOperations++ ;
	    if ( phandle -> moveCounterList == NULL ) 
		phandle -> moveCounterList = lastidxlist = pidxlist ;
	    else {
		lastidxlist -> next = pidxlist ;
		lastidxlist = pidxlist ;
	    }

	}
    }

    DB_FreeElementList (phandle -> findRequestList) ;
    phandle -> findRequestList = NULL ;

    /**** If a matching image has been found,
    ****    status is pending
    ***/

    if ( phandle -> NumberRemainOperations > 0 ) {
#ifdef DEBUG
	DB_debug(1,"DB_startMoveRequest : STATUS_Pending\n") ;
#endif
	status -> status = STATUS_Pending ;
	return (DB_NORMAL) ;
    }

    /**** else no matching image has been found,
    ****    free query identifiers list
    ****    status is success
    ***/

    else {
	phandle -> idxCounter = -1 ;
#ifdef DEBUG
	DB_debug(1,"DB_startMoveRequest : STATUS_Success\n") ;
#endif
	status -> status = STATUS_Success ;

	DB_unlock(phandle);

	return (DB_NORMAL) ;
    }


}

CONDITION 
DB_nextMoveResponse(
		DB_Handle 	*handle, 
		char 		*SOPClassUID, 
		char 		*SOPInstanceUID, 
		char 		*imageFileName, 
		unsigned short 	*numberOfRemainingSubOperations,
		DB_Status 	*status) 
{
    DB_Private_Handle	*phandle ;
    IdxRecord 		idxRec ;
    DB_CounterList		*nextlist ;

    phandle = (DB_Private_Handle *) handle ; 

    /**** If all matching images have been retrieved,
    ****    status is success
    ***/

    if ( phandle -> NumberRemainOperations <= 0 ) {
	status -> status = STATUS_Success ;

	DB_unlock(phandle);

	return (DB_NORMAL) ;
    }

    /**** Goto the next matching image number of Index File
    ***/

    if (DB_IdxRead (phandle, phandle -> moveCounterList -> idxCounter, &idxRec) != DB_NORMAL) {
#ifdef DEBUG
	DB_debug(1,"DB_nextMoveResponse : STATUS_MOVE_Failed_UnableToProcess\n") ;
#endif
	status -> status = STATUS_MOVE_Failed_UnableToProcess ;

	DB_unlock(phandle);

	return (DB_ERROR) ;
    }

    strcpy (SOPClassUID, (char *) idxRec. SOPClassUID) ;		
    strcpy (SOPInstanceUID, (char *) idxRec. SOPInstanceUID) ;		
    strcpy (imageFileName, (char *) idxRec. filename) ;

    *numberOfRemainingSubOperations = --phandle -> NumberRemainOperations ;

			
    nextlist = phandle -> moveCounterList -> next ;
    free (phandle -> moveCounterList) ;
    phandle -> moveCounterList = nextlist ;
    status -> status = STATUS_Pending ;
#ifdef DEBUG
    DB_debug(1,"DB_nextMoveResponse : STATUS_Pending\n") ;
#endif
    return (DB_NORMAL) ;
}


 
CONDITION 
DB_cancelMoveRequest (DB_Handle *handle, DB_Status *status) 
{
    DB_Private_Handle	*phandle ;
    DB_CounterList		*plist ;

    phandle = (DB_Private_Handle *) handle ; 

    while (phandle -> moveCounterList) {
	plist  = phandle -> moveCounterList ;
	phandle -> moveCounterList = phandle -> moveCounterList -> next ;
	free (plist) ;
    }
		
    status -> status = STATUS_MOVE_Cancel_SubOperationsTerminatedDueToCancelIndication ;

    DB_unlock(phandle);

    return (DB_NORMAL) ;
}



/*
** CVS Log
** dbmove.cc,v
** Revision 1.4  1996/09/27 08:46:50  hewett
** Enclosed system include files with BEGIN_EXTERN_C/END_EXTERN_C.
**
** Revision 1.3  1996/04/27 12:55:18  hewett
** Removed cause of warnings when compiled with "c++ -O -g -Wall" under
** Solaris 2.4.  Mostly due to uninitialized variables.
**
** Revision 1.2  1996/04/22 11:20:35  hewett
** Now only processes supported attributes.  Added check for presence of
** Query/Retrieve Level.
**
** Revision 1.1.1.1  1996/03/28 19:24:59  hewett
** Oldenburg Image CTN Software ported to use the dcmdata C++ toolkit.
**
**
*/
