
/*
 *
 *  Copyright (C) 1996, OFFIS
 *
 *  This software and supporting documentation were developed by
 *
 *    Kuratorium OFFIS e.V.
 *    Forschungsbereich 2: Kommunikationssysteme
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *  for CEN/TC251/WG4 as a contribution to the Computer Assisted Radiology
 *  (CAR) 1996 DICOM Demonstration.
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND OFFIS MAKES NO  WARRANTY
 *  REGARDING  THE  SOFTWARE,  ITS  PERFORMANCE,  ITS  MERCHANTABILITY  OR
 *  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES  OR
 *  ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND
 *  PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Copyright of the software  and  supporting  documentation  is,  unless
 *  otherwise stated, owned by OFFIS, and free access is hereby granted as
 *  a license to  use  this  software,  copy  this  software  and  prepare
 *  derivative works based upon this software.  However, any  distribution
 *  of this software source code or supporting documentation or derivative
 *  works  (source code and  supporting documentation)  must  include  the
 *  three paragraphs of this copyright notice.
 *
 *
 * Module: DICOM Basic Worklist Management CTN, WWW Component
 *
 * Author: Marco Eichelberg
 *
 * Purpose:
 *   Program to read one or more DICOM worklist files and pass some tags
 *   to a WWW CGI perl script (as hexadecimal encoded ASCII via stdout). 
 *   Used to display an "overview" of available worklist files.
 *
 * Last Update:      meichel
 * Update Date:      1996/06/13 09:28:52
 * CVS/RCS Revision: 1.2
 * Status:           Exp
 *
 * CVS/RCS Log
 *   readoviw.cc,v
 *   Revision 1.2  1996/06/13 09:28:52  meichel
 *   significant speedup for Worklist overview, corrected automatic date creation.
 *
 *   Revision 1.1  1996/05/31 11:43:35  meichel
 *   Initial release of the C/C++ code for the Worklist CTN WWW/CGI application.
 *
 *
 */

#include "osconfig.h"    /* make sure OS specific configuration is included first */

#include <stdio.h>
#include "dctk.h"

void printHexString(char *c)
{
  if (c) while (*c) printf("%02X",(unsigned char)(*c++));
}

void printStringElement(DcmObject& obj, DcmTagKey searchtag)
{
  DcmStack stack;
  char *c = NULL;
  if (obj.search(searchtag, stack, ESM_fromHere, FALSE ) == EC_Normal )
  {
    DcmObject *searchedObj = stack.top();
    if (searchedObj->getLength() > 0) switch(searchedObj->ident())
    {
      case EVR_AE:
      case EVR_AS:
      case EVR_CS:
      case EVR_DA:
      case EVR_DS:
      case EVR_DT:
      case EVR_IS:
      case EVR_TM:
      case EVR_UI:
        c = ((DcmByteString*)searchedObj)->get();
        break;
      case EVR_LO:
      case EVR_LT:
      case EVR_PN:
      case EVR_SH:
      case EVR_ST:
        c = ((DcmCharString*)searchedObj)->get();
        break;
      default:
        break; 
    }
  }  
  printHexString(c);
}

int main(int argc, char *argv[])
{
    if (argc < 2)
    {
      fprintf(stderr, "usage: readoviw <dicomfile> [<dicomfile>...]\n");
      return 0;
    }

    /* make sure data dictionary is loaded */
    if (dcmDataDict.numberOfEntries() == 0)
    {
      fprintf(stderr, "Warning: no data dictionary loaded, check environment variable: %s\n",
        DCM_DICT_ENVIRONMENT_VARIABLE);
      return 0;
    }

    for (int count=1; count<argc; count++)
    {
      if (count==1) printf("\\");
      DcmFileStream myin(argv[count], DCM_ReadMode);
      if ( myin.GetError() != EC_Normal )
      {
        fprintf(stderr, "readoviw: cannot open file: %s\n", argv[count]);
        printf("\\\\\\");
      } else {
        DcmFileFormat dfile;
        dfile.transferInit();
        dfile.read(myin, EXS_Unknown, EGL_withGL );
        dfile.transferEnd();

        if (dfile.error() != EC_Normal)
        {
          fprintf(stderr, "readoviw: error: %s: reading file: %s\n", 
	  	    dcmErrorConditionToString(dfile.error()), argv[count]);
        }

        DcmDataset *dataset = dfile.getDataset();
        if (dataset)
        {
          printStringElement(*dataset, DCM_AccessionNumber);
          printf("\\");
          printStringElement(*dataset, DCM_PatientName);
          printf("\\");
          printStringElement(*dataset, DCM_RequestedProcedureDescription);
          printf("\\");
        } else printf("\\\\\\");
      }
    } /* for */
}
